/* AbstractAction.java --
   Copyright (C) 2002, 2004, 2005, 2006, 2012  Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package cc.kune.common.client.actions;

import java.util.HashMap;

// TODO: Auto-generated Javadoc
/**
 * A base class for implementing the {@link Action} interface.
 * 
 * @author Andrew Selkirk
 * @author Adapted version for GWT (C) The kune development team
 */
public abstract class AbstractAction extends ChangeableObject implements Action {

  /** The Constant NO_ACTION. */
  public final static AbstractAction NO_ACTION = new NoAction();

  /**
   * Creates a new action with no properties set.
   */
  public AbstractAction() {
    // Nothing to do.
  }

  /**
   * Copy constructor used as a substitute of clone.
   *
   * @param action the action
   */
  @SuppressWarnings("unchecked")
  public AbstractAction(final AbstractAction action) {
    store = (HashMap<String, Object>) action.store.clone();
  }

  /**
   * Creates a new action with the specified name. The name is stored as a
   * property with the key {@link Action#NAME}, and no other properties are
   * initialised.
   * 
   * @param name
   *          the name (<code>null</code> permitted).
   */
  public AbstractAction(final String name) {
    // @PMD:REVIEWED:ConstructorCallsOverridableMethod: by vjrj on 21/05/09
    // 15:19
    putValue(NAME, name);
  }

  /**
   * Creates a new action with the specified name and icon. The name is stored
   * as a property with the key {@link Action#NAME}, the icon is stored as a
   * property with the key {@link Action#SMALL_ICON}, and no other properties
   * are initialised.
   * 
   * @param name
   *          the name (<code>null</code> permitted).
   * @param icon
   *          the icon (<code>null</code> permitted).
   */
  public AbstractAction(final String name, final String icon) {
    // @PMD:REVIEWED:ConstructorCallsOverridableMethod: by vjrj on 21/05/09
    // 15:19
    putValue(NAME, name);
    // @PMD:REVIEWED:ConstructorCallsOverridableMethod: by vjrj on 21/05/09
    // 15:19
    putValue(SMALL_ICON, icon);
  }

  /**
   * Returns the flag that indicates whether or not the action is enabled.
   * 
   * @return The flag.
   * 
   * @see #setEnabled(boolean)
   */
  @Override
  public boolean isEnabled() {
    return (Boolean) super.getValue(ENABLED);
  }

  /**
   * On attach.
   */
  public void onAttach() {
  }

  /**
   * On dettach.
   */
  public void onDettach() {
  }

  /* (non-Javadoc)
   * @see cc.kune.common.client.actions.ChangeableObject#putValue(java.lang.String, java.lang.Object)
   */
  @Override
  /**
   * Sets the value associated with the specified key and sends a
   * {@link java.beans.PropertyChangeEvent} to all registered listeners. The
   * standard keys are:
   * <ul>
   * <li>{@link #NAME}</li>
   * <li>{@link #SHORT_DESCRIPTION}</li>
   * <li>{@link #LONG_DESCRIPTION}</li>
   * <li>{@link #SMALL_ICON}</li>
   * <li>{@link #ACTION_COMMAND_KEY}</li>
   * <li>{@link #ACCELERATOR_KEY}</li>
   * <li>{@link #MNEMONIC_KEY}</li>
   * </ul>
   * Any existing value associated with the key will be overwritten.
   *
   * @param key
   *            the key (not <code>null</code>).
   * @param value
   *            the value (<code>null</code> permitted).
   */
  public void putValue(final String key, final Object value) {
    super.putValue(key, value);
  }

  /**
   * Sets the flag that indicates whether or not the action is enabled and, if
   * the value of the flag changed from the previous setting, sends a.
   *
   * @param enabled the new flag value.
   * {@link java.beans.PropertyChangeEvent} to all registered listeners (using
   * the property name 'enabled').
   * @see #isEnabled()
   */
  @Override
  public void setEnabled(final boolean enabled) {
    putValue(ENABLED, enabled);
  }

  /**
   * Sets the shortcut.
   *
   * @param key the new shortcut
   */
  public void setShortcut(final KeyStroke key) {
    putValue(Action.ACCELERATOR_KEY, key);
  }
}
